/* SymbolT.java - zapouzdruje tabulku symbolu.
 *
 * Michal Beran, berny@students.zcu.cz
 * 4.1.2001
 */

package pl0prog;

import java.io.*;

/** Trida SymbolT implementuje tabulku symbolu pouzivanou prekladacem
  * a interpretem jazyka PL/0.
  */
public class SymbolT{

	/** Maximalni pocet identifikatoru v tabulce
	  */
	private final int TX_MAX = 5000; 
	/** Nejvyssi mozna adesa
	  */
	private final int AMAX = 2048;
	/** Tabulka identifikatoru
	  */
	private Identifier[] table = new Identifier[TX_MAX];
	/** Index ukazujici na nasleduji volny prvek
	  */
	private int tableIndex = 0;

	/** Vyhleda symbol v tabulce symbolu
	  * @param id  jmeno symbolu,
	  * @return  -1: symbol nenalezen, >=0 : adresa symbolu.
	  */
	protected int position(String id) {
		int i;
		table[0].name = new String(id);
		i = tableIndex;
		while (table[i].name.compareTo(id) != 0)
			i--;
		return i;
	}

	/** Vlozeni konstanty do tabulky symbolu.
	  * @param identifier  identifikator konstanty,
	  * @param value       hodnota konstanty,
	  */
	protected void enterConstant(String identifier, int value) {
		tableIndex++;
		table[tableIndex] = new Constant(identifier, value);
	}

	/** Vlozeni promenne do tabulky symbolu.
	  * @param identifier  identifikator promenne,
	  * @param address     adresa,
	  * @param level       uroven zanoreni promenne.
	  */
	protected void enterVariable(String identifier, int address, int level) {
		tableIndex++;
		table[tableIndex] = new Variable(identifier, address, level);
	}

	/** Vlozeni procedury do tabulky symbolu 
	  * @param identifier  identifikator procedury,
	  * @param level       uroven zanoreni procedury,
	  */
	protected void enterProcedure(String identifier, int level) {
		tableIndex++;
		table[tableIndex] = new Procedure(identifier, level);
	}

	/** Nastaveni adresy symbolu.
	  * @param tx  offset symbolu v tabulce,
	  * @param cx  nova hodnota adresy.
	  */
	protected void setAddr(int tx, int cx) {
		if (table[tx] == null) {
			table[tx] = new Variable("", cx, 0);
		} else if (table[tx] instanceof Locatable) {
			((Locatable) table[tx]).setAddress(cx);
		}
	}

	/** Ziskani prislusneho identifikatoru z tabulky identifikatoru 
 	  * @param i  offset symbolu.
	  * @return  vraceny symbol.
	  */
	protected Identifier getIdentifier(int i) {
		return table[i];
	}

	/** Vraci index mista, kde je ulozen posledni vlozeny identifikator.
	  * @return  index posledniho identifikatoru.
	  */
	protected int getTableIndex() {
		return tableIndex;
	}

	/** Metoda pro vypis kon. tvaru tabulky symbolu do souboru; generuje 
	  * vyjimku IOException.
	  * @param path  soubor, kam ma byt vypis tabulky symbolu ulozen.
	  */
	public void list(String path) throws IOException {
		try {
			FileWriter file = new FileWriter(path);
			BufferedWriter out = new BufferedWriter(file);
			out.write("\n\nTabulka symbolu:\n");
			out.write("***************\n");
			for (int i = 1; i <= tableIndex; i++) {
				out.write(i + " ");
				out.write(table[i].toString());
				out.newLine();
			}
			out.close();
		} catch (IOException e) {
			System.out.println("File " + path + " couldn't be created.");
			throw e;
		}
	}

}
